---- CombustionTracker -- made by Tracerus on Wrathbringer (EU) ----------

local CombustionTracker_Version = "1.21";
local CombustionTracker_VariablesLoaded = false;
local CombustionTracker_TextureName = "(.+)SealOfFire";
local CombustionTracker_maxCrits = 3;
CombustionTracker_CritsLeft = 0; --not local to allow saving/loading


---------------------
--   debug stuff   --
---------------------
local CombustionTracker_DebugMode = false;
function printdebug(msg)
	if( CombustionTracker_DebugMode and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[CombustionTracker]: "..msg, 1, 1, 0);
	end
end

---------------------------------------------
--					handles all events						 --
---------------------------------------------
function CombustionTracker_OnEvent()
	
	if( event=="VARIABLES_LOADED") then
		CombustionTracker_VariablesLoaded = true;
		
		--register all events we need
		this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
		this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF");
		this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
		this:RegisterEvent("PLAYER_AURAS_CHANGED");
		this:RegisterEvent("PLAYER_DEAD");
		
		--if combustion was active when logged out (so critsleft is more than zero)
		if( CombustionTracker_CritsLeft>0 ) then
		
			CombustionTracker_UpdateText();
			if( CombustionTracker_UpdateTextPosition() ) then
				CombustionTracker_MainFrame_CritNumber:Show();
			end
		
			--if we use Satrina mode we have to hook the OnEvent function to notice the creation of the buff buttons
			--and to be able to show our text right after login (else there would be no buffs fom Satrina even after
			--the first PLAYER_AURA_CHANGED event is fired which is used for first setup of the existing buffs)
			if( getglobal("SatrinaBuffFrame") )then
				--hook Satrina's BuffButton_OnEvent function
				local oldSatOnEvent = SatrinaBuffFrame_BuffButton_OnEvent;
				SatrinaBuffFrame_BuffButton_OnEvent = function(event)
					--call old OnEvent function
					oldSatOnEvent(event);
					--try to show our text and unhook OnEvent of
					if( CombustionTracker_UpdateTextPosition() ) then
						CombustionTracker_MainFrame_CritNumber:Show();
						SatrinaBuffFrame_BuffButton_OnEvent = oldSatOnEvent;
					end

				end
			
			end
		end
		
	end
	
	if( not CombustionTracker_VariablesLoaded ) then
		return;
	end
	
	--if we got a buff and system message is equal to combustion gain message ...
	if( event=="CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS" and arg1==COMBUSTIONTRACKER_AURAGAIN ) then
		--we got combustion, so start listening to needed messages
		this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
		this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF");
		this:RegisterEvent("PLAYER_AURAS_CHANGED"); --this event will be triggered right after CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS and is used to position the text and show it
		this:RegisterEvent("PLAYER_DEAD");
		--stop listening to buff gain messages
		this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
		
		CombustionTracker_CritsLeft = CombustionTracker_maxCrits;
		
		CombustionTracker_UpdateText();
		--CombustionTracker_UpdateTextPosition();
		--CombustionTracker_MainFrame_CritNumber:Show();
		
	--...else if a buff is gone and system message is equal to combustion fade message        OR if we died ...
	elseif( (event=="CHAT_MSG_SPELL_AURA_GONE_SELF" and arg1==COMBUSTIONTRACKER_AURALOSE) or event=="PLAYER_DEAD" ) then
		CombustionTracker_MainFrame_CritNumber:Hide();

		--combustion ended, so stop listening to messages
		this:UnregisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
		this:UnregisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF");
		this:UnregisterEvent("PLAYER_AURAS_CHANGED");
		this:UnregisterEvent("PLAYER_DEAD");
		--start listening to buff gain messages again
		this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");

	--...else if did we just do damage...
	elseif( event=="CHAT_MSG_SPELL_SELF_DAMAGE" ) then
		--if we had a critical hit                       and it is a firespell
		if( string.find(arg1, COMBUSTIONTRACKER_CRITHIT) and IsFirespellMessage(arg1) ) then
			--decrease number of crits left
			CombustionTracker_CritsLeft = CombustionTracker_CritsLeft-1;

			--sometimes it takes a while until buff fades, so hide the UI text if there are 0 crits left
			if( CombustionTracker_CritsLeft<=0 ) then
				CombustionTracker_MainFrame_CritNumber:Hide();
			--else update text
			else
				CombustionTracker_UpdateText();
			end
		end

	--... else if any buff changed we maybe have to reposition the UI text
	elseif( event=="PLAYER_AURAS_CHANGED" ) then
		--...so we update it
		if( CombustionTracker_UpdateTextPosition() ) then
			CombustionTracker_MainFrame_CritNumber:Show();
		end
	end

end

-------------------------------------------------------
--	updates the text position for standard buff bar  --
-------------------------------------------------------
function CombustionTracker_UpdateTextPosition()
	--go through all the buffs 
	for i = 0, 23 do
		local buffIndex, untilCancelled = GetPlayerBuff(i, "HELPFUL");
		--test if we found a buff and get its texturename
		if ( buffIndex >= 0 ) then
			local buffTexture = GetPlayerBuffTexture(buffIndex);	
			
			--test if it is combustion texture
			if (string.find(buffTexture, CombustionTracker_TextureName)) then
			
				--generate tooltip for this buff and extract buffname
				CombustionTracker_Tooltip:SetOwner(UIParent, "ANCHOR_NONE");
				CombustionTracker_Tooltip:SetPlayerBuff(buffIndex);
				local buffName = tostring(CombustionTracker_TooltipTextLeft1:GetText());
				CombustionTracker_Tooltip:Hide();
				
				--check if this is really combustion buff (and not Fire Resistance Aura)
				if ( buffName and COMBUSTIONTRACKER_COMBUSTION==buffName ) then
					--place UI text under combustion buff and update text
					CombustionTracker_MainFrame_CritNumber:SetPoint("TOP", "BuffButton"..i, "BOTTOM", -2, 0);
					--CombustionTracker_MainFrame_CritNumber:Show();

					--we found combustion buff and can stop going through bufflist
					return true;
				end
			end
		end
	end
	--no combustion buff found -> hide text
	CombustionTracker_MainFrame_CritNumber:Hide();
	return false;
end

-------------------------------------------------------
--    updates the text position for ElkanoBuffBar    --
-------------------------------------------------------
function CombustionTracker_UpdateTextPosition_ElkBuffBar()
	--go through all the buffs 
	for i = 1, 24 do
		--get buff name directly out of ElkBuffBars
		local buffName = getglobal("ElkBuffButton"..i.."DescribeText");
		if ( buffName and COMBUSTIONTRACKER_COMBUSTION==tostring(buffName:GetText()) ) then
			--set button as parent for scaling and anchor text to it
			CombustionTracker_MainFrame_CritNumber:SetParent("ElkBuffButton"..i);
			CombustionTracker_MainFrame_CritNumber:SetPoint("RIGHT", "ElkBuffButton"..i, "RIGHT", -2, 0);
			--CombustionTracker_MainFrame_CritNumber:Show();
			
			--we found combustion buff and can stop going through bufflist
			return true;
		end
	end
	--no combustion buff found -> hide text
	CombustionTracker_MainFrame_CritNumber:Hide();
	return false;
end

--------------------------------------------------------
--   updates the text position for SatrinaBuffFrame   --
--------------------------------------------------------
function CombustionTracker_UpdateTextPosition_SatrinaBuff()
	--go through all the Satrina buff buttons
	for i = 1, 24 do
		--get a button
		local buffButton = getglobal("SatrinaBuffButton"..i);
		--extract buff name out of buttons tooltip
		if( buffButton and buffButton.index ) then
			--generate tooltip for this buff and extract buffname
			CombustionTracker_Tooltip:SetOwner(UIParent, "ANCHOR_NONE");
			CombustionTracker_Tooltip:SetPlayerBuff(buffButton.index);
			local buffName = tostring(CombustionTracker_TooltipTextLeft1:GetText());
			CombustionTracker_Tooltip:Hide();
			
			--check if it is combustion buff
			if ( buffName and COMBUSTIONTRACKER_COMBUSTION==buffName ) then
				--set button as parent for scaling and anchor text to it
				CombustionTracker_MainFrame_CritNumber:SetParent("SatrinaBuffButton"..i.."Duration");
				CombustionTracker_MainFrame_CritNumber:SetPoint("TOP", buffButton, "BOTTOM", 0, 0);
				--CombustionTracker_MainFrame_CritNumber:Show();
				
				--we found combustion buff and can stop going through bufflist
				return true;
			end
		end
	end
	--no combustion buff found -> hide text
	CombustionTracker_MainFrame_CritNumber:Hide();
	return false;
end
--]]

------------------------------
--   updates text content   --
------------------------------
function CombustionTracker_UpdateText()
	CombustionTracker_MainFrame_CritNumber:SetText(CombustionTracker_CritsLeft);
end

-------------------------------------------------------------
--   checks if a given message contains a fire spellname   --
-------------------------------------------------------------
function IsFirespellMessage(msg)
	local i=1;
	--go through the list of firespells and return true if one is found in the message
	while( COMBUSTIONTRACKER_FIRESPELLS[i] ) do
		if( string.find(msg, COMBUSTIONTRACKER_FIRESPELLS[i]) ) then
			return true;
		end
		i = i+1;
	end
	
	--else return false
	return false;
end

-------------------------------------
--   called when addon is loaded   --
-------------------------------------
function CombustionTracker_OnLoad()
	--disable this mod if we are not a mage (not really disable but just do nothing)
	if (not (COMBUSTIONTRACKER_MAGE == UnitClass("player"))) then
		DEFAULT_CHAT_FRAME:AddMessage(COMBUSTIONTRACKER_DISABLED);
		return;
	end
	
	if(myAddOnsFrame_Register) then
		local details = {
			name = "CombustionTracker",
			version = CombustionTracker_Version,
			releaseDate = "September 02, 2006",
			author = "Mario Saban (Tracerus @ Wrathbringer(EU))",
			email = "mttracer@gmx.de",
			website = "http://www.curse-gaming.com/de/wow/addons-5040-1-combustiontracker.html",
			category = MYADDONS_CATEGORY_CLASS,
			--optionsframe = "HelloWorldOptionsFrame"
		};
	
		myAddOnsFrame_Register(details); 
	end
	
	local welcome = "CombustionTracker v"..CombustionTracker_Version.." "..COMBUSTIONTRACKER_LOADED;
	
	--test for supported buff bar addons and change positioning function and some font attributes to match the addon ones
	--ElkBuffBar
	if( getglobal("ElkBuffBarFrame") ) then
		CombustionTracker_UpdateTextPosition = CombustionTracker_UpdateTextPosition_ElkBuffBar;
		welcome = welcome.." Switched to ElkBuffBar mode.";
		
		CombustionTracker_MainFrame_CritNumber:SetFontObject(GameFontNormal);
		CombustionTracker_MainFrame_CritNumber:SetTextColor(1, 1, 1);
		CombustionTracker_MainFrame_CritNumber:SetJustifyH("RIGHT");

	--SatrinaBuffFrame
	elseif( getglobal("SatrinaBuffFrame") ) then
		CombustionTracker_UpdateTextPosition = CombustionTracker_UpdateTextPosition_SatrinaBuff;
		welcome = welcome.." Switched to SatrinaBuffFrame mode.";
		
		--CombustionTracker_MainFrame_CritNumber:SetFontObject(GameFontNormalSmall);
		--CombustionTracker_MainFrame_CritNumber:SetVertexColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		--set parent to any Duration text to let is scale automatically		
		CombustionTracker_MainFrame_CritNumber:SetParent("SatrinaBuffButton1Duration");
	end
	
	--welcome message
	DEFAULT_CHAT_FRAME:AddMessage(welcome, 1, 1, 0);

	--wait for saved variables loaded
	this:RegisterEvent("VARIABLES_LOADED");
end